package pomodoro

import (
	"git.smsvc.net/pomodoro/GoTomato/internal/shared"
	"sync"
	"time"
)

var mu sync.Mutex // to synchronize access to shared state
var timer Timer

func waitForTimer(t Timer) bool {
	for {
		select {
		case <-t.End:
			return true
		case <-t.Abort:
			return false
		case shared.Message.TimeLeft = <-t.TimeLeft:
			// do nothing, just let the timer update the message
		}
	}
}

// RunPomodoro iterates the Pomodoro work/break sessions.
func RunPomodoro() {
	mu.Lock()
	shared.Message.Ongoing = true
	shared.Message.Paused = false
	mu.Unlock()

	pomodoroConfig := shared.Message.Settings

	for session := 1; session <= pomodoroConfig.Sessions; session++ {
		timer = timer.Init()

		shared.Message.Session = session

		shared.Message.Mode = "Work"
		go timer.Start(pomodoroConfig.Work)
		if !waitForTimer(timer) {
			break
		}

		if session < pomodoroConfig.Sessions {
			shared.Message.Mode = "ShortBreak"
			go timer.Start(pomodoroConfig.ShortBreak)
			if !waitForTimer(timer) {
				break
			}
		} else { // last phase, prepare for finish
			shared.Message.Mode = "LongBreak"
			go timer.Start(pomodoroConfig.LongBreak)
			if !waitForTimer(timer) {
				break
			}
			shared.Message.Mode = "End"
			time.Sleep(time.Second)
		}
	}

	mu.Lock()
	shared.Message.Ongoing = false
	shared.Message.Paused = false
	mu.Unlock()

	shared.Message.Mode = "Idle"
	shared.Message.Session = 0
	shared.Message.TimeLeft = shared.Message.Settings.Work
}

func ResetPomodoro() {
	timer.Stop()
}

func PausePomodoro() {
	mu.Lock()
	shared.Message.Paused = true
	mu.Unlock()

	timer.Pause()
}

func ResumePomodoro() {
	mu.Lock()
	shared.Message.Paused = false
	mu.Unlock()
	timer.Resume()
}

func IsPomodoroOngoing() bool {
	mu.Lock()
	defer mu.Unlock() // Ensures that the mutex is unlocked after the function is done
	return shared.Message.Ongoing
}

func IsPomodoroPaused() bool {
	mu.Lock()
	defer mu.Unlock() // Ensures that the mutex is unlocked after the function is done
	return shared.Message.Paused
}
